# -*- coding: utf-8 -*-

from typing import Any

from qtpy import QtWidgets

from prettyqt.utils import bidict, InvalidParamError

QScrollerProperties = QtWidgets.QScrollerProperties

FRAME_RATES = bidict(
    fps_60=QScrollerProperties.Fps60,
    fps_30=QScrollerProperties.Fps30,
    fps_20=QScrollerProperties.Fps20,
    standard=QScrollerProperties.Standard,
)

OVERSHOOT_POLICY = bidict(
    when_scrollable=QScrollerProperties.OvershootWhenScrollable,
    always_off=QScrollerProperties.OvershootAlwaysOff,
    always_on=QScrollerProperties.OvershootAlwaysOn,
)

SCROLL_METRIC = bidict(
    mouse_press_event_delay=QScrollerProperties.MousePressEventDelay,
    drag_start_distance=QScrollerProperties.DragStartDistance,
    drag_velocity_smoothing_factor=QScrollerProperties.DragVelocitySmoothingFactor,
    axis_lock_threshold=QScrollerProperties.AxisLockThreshold,
    scrolling_curve=QScrollerProperties.ScrollingCurve,
    deceleration_factor=QScrollerProperties.DecelerationFactor,
    minimum_velocity=QScrollerProperties.MinimumVelocity,
    maximum_velocity=QScrollerProperties.MaximumVelocity,
    maximum_click_through_velocity=QScrollerProperties.MaximumClickThroughVelocity,
    accelerating_flick_maximum_time=QScrollerProperties.AcceleratingFlickMaximumTime,
    accelerating_flick_speedup_factor=QScrollerProperties.AcceleratingFlickSpeedupFactor,
    snap_position_ratio=QScrollerProperties.SnapPositionRatio,
    snap_time=QScrollerProperties.SnapTime,
    overshoot_drag_resistance_factor=QScrollerProperties.OvershootDragResistanceFactor,
    overshoot_drag_distance_factor=QScrollerProperties.OvershootDragDistanceFactor,
    overshoot_scroll_distance_factor=QScrollerProperties.OvershootScrollDistanceFactor,
    overshoot_scroll_time=QScrollerProperties.OvershootScrollTime,
    horizontal_overshoot_policy=QScrollerProperties.HorizontalOvershootPolicy,
    vertical_overshoot_policy=QScrollerProperties.VerticalOvershootPolicy,
    frame_rate=QScrollerProperties.FrameRate,
    scroll_metric_count=QScrollerProperties.ScrollMetricCount,
)


class ScrollerProperties(QtWidgets.QScrollerProperties):
    def __getitem__(self, metric: str):
        return self.get_scroll_metric(metric)

    def __setitem__(self, metric: str, value: Any):
        self.set_scroll_metric(metric, value)

    def set_scroll_metric(self, metric: str, value: Any):
        """Set scroll metric.

        Possible values: "mouse_press_event_delay", "drag_start_distance",
                         "drag_velocity_smoothing_factor", "axis_lock_threshold",
                         "scrolling_curve", "deceleration_factor", "minimum_velocity",
                         "maximum_velocity", "maximum_click_through_velocity",
                         "accelerating_flick_maximum_time",
                         "accelerating_flick_speedup_factor", "snap_position_ratio",
                         "snap_time", "overshoot_drag_resistance_factor",
                         "overshoot_drag_distance_factor",
                         "overshoot_scroll_distance_factor", "overshoot_scroll_time",
                         "horizontal_overshoot_policy", "vertical_overshoot_policy",
                         "frame_rate", "scroll_metric_count"

        Args:
            metric: Scroll metric to set
            value: Value to set

        Raises:
            InvalidParamError: metric does not exist

        """
        if metric not in SCROLL_METRIC:
            raise InvalidParamError(metric, SCROLL_METRIC)
        self.setScrollMetric(SCROLL_METRIC[metric], value)

    def get_scroll_metric(self, metric: str) -> Any:
        """Return scroll metric.

        Possible values: "mouse_press_event_delay", "drag_start_distance",
                         "drag_velocity_smoothing_factor", "axis_lock_threshold",
                         "scrolling_curve", "deceleration_factor", "minimum_velocity",
                         "maximum_velocity", "maximum_click_through_velocity",
                         "accelerating_flick_maximum_time",
                         "accelerating_flick_speedup_factor", "snap_position_ratio",
                         "snap_time", "overshoot_drag_resistance_factor",
                         "overshoot_drag_distance_factor",
                         "overshoot_scroll_distance_factor", "overshoot_scroll_time",
                         "horizontal_overshoot_policy", "vertical_overshoot_policy",
                         "frame_rate", "scroll_metric_count"

        Args:
            metric: Scroll metric to get

        Raises:
            InvalidParamError: metric does not exist

        Returns:
            state
        """
        if metric not in SCROLL_METRIC:
            raise InvalidParamError(metric, SCROLL_METRIC)
        return self.scrollMetric(SCROLL_METRIC[metric])



# -*- coding: utf-8 -*-

from qtpy import QtCore


class RegExp(QtCore.QRegExp):
    def __repr__(self):
        return f"RegExp({self.pattern()!r})"

    def __reduce__(self):
        return (self.__class__, (self.pattern(),))


if __name__ == "__main__":
    reg = RegExp()
    reg.setPattern("[0-9]")



# -*- coding: utf-8 -*-

"""svg module.

contains QtSvg-based classes
"""

from .graphicssvgitem import GraphicsSvgItem
from .svggenerator import SvgGenerator


__all__ = ["GraphicsSvgItem", "SvgGenerator"]



# -*- coding: utf-8 -*-

from qtpy import QtWidgets

from prettyqt import widgets


QtWidgets.QStyleOptionButton.__bases__ = (widgets.StyleOption,)


class StyleOptionButton(QtWidgets.QStyleOptionButton):
    pass



# -*- coding: utf-8 -*-

from qtpy import QtWidgets, QtCore

from prettyqt import core
from prettyqt.utils import bidict, InvalidParamError

ORIENTATIONS = bidict(horizontal=QtCore.Qt.Horizontal, vertical=QtCore.Qt.Vertical)

SUBMIT_POLICIES = bidict(
    auto=QtWidgets.QDataWidgetMapper.AutoSubmit,
    manual=QtWidgets.QDataWidgetMapper.ManualSubmit,
)

QtWidgets.QDataWidgetMapper.__bases__ = (core.Object,)


class DataWidgetMapper(QtWidgets.QDataWidgetMapper):
    def serialize_fields(self):
        return dict(
            current_index=self.currentIndex(),
            orientation=self.get_orientation(),
            submit_policy=self.get_submit_policy(),
        )

    def set_orientation(self, orientation: str):
        """Set the orientation of the splitter.

        Allowed values are "horizontal", "vertical"

        Args:
            orientation: orientation for the splitter

        Raises:
            InvalidParamError: orientation does not exist
        """
        if orientation not in ORIENTATIONS:
            raise InvalidParamError(orientation, ORIENTATIONS)
        self.setOrientation(ORIENTATIONS[orientation])

    def get_orientation(self) -> str:
        """Return current orientation.

        Possible values: "horizontal", "vertical"

        Returns:
            orientation
        """
        return ORIENTATIONS.inv[self.orientation()]

    def set_submit_policy(self, policy: str):
        """Set the submit policy of the mapper.

        Allowed values are "auto", "manual"

        Args:
            policy: submit_policy for the splitter

        Raises:
            InvalidParamError: submit_policy does not exist
        """
        if policy not in SUBMIT_POLICIES:
            raise InvalidParamError(policy, SUBMIT_POLICIES)
        self.setSubmitPolicy(SUBMIT_POLICIES[policy])

    def get_submit_policy(self) -> str:
        """Return current submit policy.

        Possible values: "auto", "manual"

        Returns:
            submit policy
        """
        return SUBMIT_POLICIES.inv[self.submitPolicy()]



# -*- coding: utf-8 -*-

from qtpy import QtCore

from prettyqt import core


QtCore.QIdentityProxyModel.__bases__ = (core.AbstractProxyModel,)


class IdentityProxyModel(QtCore.QIdentityProxyModel):
    pass



# -*- coding: utf-8 -*-

from typing import Tuple, Optional, Union

from qtpy import PYQT5, PYSIDE2

if PYQT5:
    from PyQt5 import QtPositioning
elif PYSIDE2:
    from PySide2 import QtPositioning


from prettyqt import positioning

QtPositioning.QGeoCircle.__bases__ = (positioning.GeoShape,)


class GeoCircle(QtPositioning.QGeoCircle):
    def __init__(
        self,
        center_or_other: Optional[
            Union[
                QtPositioning.QGeoShape, QtPositioning.QGeoCoordinate, Tuple[float, float]
            ]
        ] = None,
        radius: Optional[float] = None,
    ):
        if center_or_other is None:
            super().__init__()
        else:
            if radius is None:
                radius = -1
            if isinstance(center_or_other, tuple):
                center_or_other = QtPositioning.QGeoCoordinate(*center_or_other)
            super().__init__(center_or_other, radius)

    def __repr__(self):
        return f"GeoCircle({self.get_center()!r}, {self.radius()})"

    def get_center(self) -> positioning.GeoCoordinate:
        return positioning.GeoCoordinate(self.center())


if __name__ == "__main__":
    coord = (1, 1)
    circle = GeoCircle(coord)
    print(str(circle))
    print(repr(circle))



# -*- coding: utf-8 -*-

from qtpy import QtWidgets

from prettyqt import core

QtWidgets.QKeyEventTransition.__bases__ = (core.EventTransition,)


class KeyEventTransition(QtWidgets.QKeyEventTransition):
    pass



# -*- coding: utf-8 -*-

from qtpy import QtCore

from prettyqt import core

QtCore.QSignalTransition.__bases__ = (core.AbstractTransition,)


class SignalTransition(QtCore.QSignalTransition):
    pass



# -*- coding: utf-8 -*-

from __future__ import annotations

from qtpy import QtCore


class UrlQuery(QtCore.QUrlQuery):
    def __repr__(self):
        return f"UrlQuery({self.toString()!r})"

    def __str__(self):
        return self.toString()

    def __contains__(self, key: str):
        return self.hasQueryItem(key)

    def __add__(self, other: dict) -> UrlQuery:
        for k, v in other.items():
            self.addQueryItem(k, str(v))
        return self

    def serialize_fields(self):
        return dict(path=self.toString())

    def serialize(self):
        return self.serialize_fields()

    # def add_query_items(self, **items: str):
    #     for k, v in items.items():
    #         self.addQueryitem(k, v)



from qtpy import QtHelp

from prettyqt import core


class HelpContentItem(QtHelp.QHelpContentItem):
    def __len__(self):
        return self.childCount()

    def get_url(self) -> core.Url:
        return core.Url(self.url())



# -*- coding: utf-8 -*-

from qtpy.QtCharts import QtCharts

from prettyqt import charts


QtCharts.QBarSeries.__bases__ = (charts.AbstractBarSeries,)


class BarSeries(QtCharts.QBarSeries):
    pass



# -*- coding: utf-8 -*-

from qtpy import QtCore

from prettyqt import core
from prettyqt.utils import bidict, InvalidParamError


TRANSITION_TYPES = bidict(
    exclusive=QtCore.QAbstractTransition.ExternalTransition,
    parallel=QtCore.QAbstractTransition.InternalTransition,
)


QtCore.QAbstractTransition.__bases__ = (core.Object,)


class AbstractTransition(QtCore.QAbstractTransition):
    def set_transition_type(self, typ: str):
        """Set transition type.

        Allowed values are "exclusive", "parallel"

        Args:
            typ: transition type to use

        Raises:
            InvalidParamError: transition type does not exist
        """
        if typ not in TRANSITION_TYPES:
            raise InvalidParamError(typ, TRANSITION_TYPES)
        self.setTransitionType(TRANSITION_TYPES[typ])

    def get_transition_type(self) -> str:
        """Return current transition type.

        Possible values: "exclusive", "parallel"

        Returns:
            transition type
        """
        return TRANSITION_TYPES.inv[self.transitionType()]



from qtpy import QtWidgets

from prettyqt import widgets, core


QtWidgets.QGraphicsLineItem.__bases__ = (widgets.GraphicsItem,)


class GraphicsLineItem(QtWidgets.QGraphicsLineItem):
    def __repr__(self):
        return f"{self.__class__.__name__}({repr(self.get_line())})"

    def get_line(self):
        return core.LineF(self.line())


if __name__ == "__main__":
    item = GraphicsLineItem(core.Point(0, 0), core.Point(2, 2))
    print(repr(item))



# -*- coding: utf-8 -*-

from typing import List

from qtpy import QtLocation

from prettyqt import location


class PlaceMatchRequest(QtLocation.QPlaceMatchRequest):
    def __setitem__(self, index: str, val):
        attrs = self.parameters()
        attrs[index] = val
        self.setParameters(attrs)

    def __getitem__(self, index: str):
        attr = self.parameters()
        if index not in attr:
            raise KeyError(f"Key {index!r} does not exist.")
        return attr[index]

    def get_places(self) -> List[location.Place]:
        return [location.Place(i) for i in self.places()]


if __name__ == "__main__":
    request = PlaceMatchRequest()



# -*- coding: utf-8 -*-

from qtpy import QtWidgets

from prettyqt import widgets


QtWidgets.QStyleOptionFrame.__bases__ = (widgets.StyleOption,)


class StyleOptionFrame(QtWidgets.QStyleOptionFrame):
    pass



# -*- coding: utf-8 -*-

from qtpy.QtCharts import QtCharts

from prettyqt import charts

QtCharts.QLogValueAxis.__bases__ = (charts.AbstractAxis,)


class LogValueAxis(QtCharts.QLogValueAxis):
    pass



# -*- coding: utf-8 -*-

from typing import List

from pygments.lexer import RegexLexer, bygroups
from pygments.token import Comment, Keyword, Name, Number, Operator, Text, Token


class CustomLexer(RegexLexer):
    name = "regex"
    aliases = ["regex"]
    filenames: List[str] = []

    tokens = {
        "root": [
            (r"\w+", Name),
            (r"\d+", Number),
            (r"[\s\,\:\-\"\']+", Text),
            (r"[\$\^]", Token),
            (r"[\+\*\.\?]", Operator),
            (
                r"(\()([\?\<\>\!\=\:]{2,3}.+?)(\))",
                bygroups(Keyword.Namespace, Name.Function, Keyword.Namespace),
            ),
            (r"(\()(\?\#.+?)(\))", bygroups(Comment, Comment, Comment)),
            (r"[\(\)]", Keyword.Namespace),
            (r"[\[\]]", Name.Class),
            (r"\\\w", Keyword),
            (r"[\{\}]", Operator),
        ]
    }



# -*- coding: utf-8 -*-

"""Multimedia module."""

from .mediatimeinterval import MediaTimeInterval
from .mediatimerange import MediaTimeRange
from .abstractvideobuffer import AbstractVideoBuffer

# from .abstractplanarvideobuffer import AbstractPlanarVideoBuffer
from .videoframe import VideoFrame
from .audioformat import AudioFormat
from .camerainfo import CameraInfo
from .camerafocuszone import CameraFocusZone
from .camerafocus import CameraFocus
from .cameraexposure import CameraExposure
from .cameraimageprocessing import CameraImageProcessing
from .mediabindableinterface import MediaBindableInterface
from .imageencodersettings import ImageEncoderSettings
from .videoencodersettings import VideoEncoderSettings
from .audioencodersettings import AudioEncoderSettings
from .mediaobject import MediaObject
from .camera import Camera
from .cameraviewfindersettings import CameraViewfinderSettings
from .mediacontent import MediaContent
from .mediacontrol import MediaControl
from .mediaplaylist import MediaPlaylist
from .mediaplayer import MediaPlayer
from .mediarecorder import MediaRecorder
from .audiorecorder import AudioRecorder
from .soundeffect import SoundEffect

__all__ = [
    "AudioFormat",
    "AbstractVideoBuffer",
    "ImageEncoderSettings",
    # "AbstractPlanarVideoBuffer",
    "VideoFrame",
    "CameraInfo",
    "CameraFocusZone",
    "CameraFocus",
    "CameraExposure",
    "CameraImageProcessing",
    "CameraViewfinderSettings",
    "Camera",
    "MediaBindableInterface",
    "VideoEncoderSettings",
    "AudioEncoderSettings",
    "MediaControl",
    "MediaContent",
    "MediaObject",
    "MediaPlaylist",
    "MediaPlayer",
    "MediaRecorder",
    "MediaTimeInterval",
    "MediaTimeRange",
    "AudioRecorder",
    "SoundEffect",
]



# -*- coding: utf-8 -*-

"""Top-level package for PrettyQt."""

__author__ = """Philipp Temminghoff"""
__email__ = "phil65@kodi.tv"
__version__ = "0.143.1"


def debug():
    """Print the local variables in the caller's frame."""
    import inspect

    frame = inspect.currentframe()
    try:
        from prettyqt.objbrowser import objectbrowser

        objectbrowser.ObjectBrowser.browse(frame.f_back.f_globals)
    finally:
        del frame


__all__ = [
    "core",
    "gui",
    "widgets",
    "custom_widgets",
    "custom_models",
    "syntaxhighlighters",
    "charts",
    "webenginewidgets",
    "debug",
]



# -*- coding: utf-8 -*-
from __future__ import annotations

from typing import List

from qtpy import QtNetwork

from prettyqt import network
from prettyqt.utils import bidict

INTERFACE_FLAGS = bidict(
    is_up=QtNetwork.QNetworkInterface.IsUp,
    is_running=QtNetwork.QNetworkInterface.IsRunning,
    can_broadcast=QtNetwork.QNetworkInterface.CanBroadcast,
    is_loop_back=QtNetwork.QNetworkInterface.IsLoopBack,
    is_point_to_point=QtNetwork.QNetworkInterface.IsPointToPoint,
    can_multicast=QtNetwork.QNetworkInterface.CanMulticast,
)

INTERFACE_TYPES = bidict(
    unknown=QtNetwork.QNetworkInterface.Unknown,
    loopback=QtNetwork.QNetworkInterface.Loopback,
    virtual=QtNetwork.QNetworkInterface.Virtual,
    ethernet=QtNetwork.QNetworkInterface.Ethernet,
    wifi=QtNetwork.QNetworkInterface.Wifi,
    can_bus=QtNetwork.QNetworkInterface.CanBus,
    fddi=QtNetwork.QNetworkInterface.Fddi,
    ppp=QtNetwork.QNetworkInterface.Ppp,
    slip=QtNetwork.QNetworkInterface.Slip,
    phonet=QtNetwork.QNetworkInterface.Phonet,
    ieee802154=QtNetwork.QNetworkInterface.Ieee802154,
    sixlowpan=QtNetwork.QNetworkInterface.SixLoWPAN,
    ieee80216=QtNetwork.QNetworkInterface.Ieee80216,
    ieee1394=QtNetwork.QNetworkInterface.Ieee1394,
)


class NetworkInterface(QtNetwork.QNetworkInterface):
    # def __bool__(self):
    #     return self.isValid()

    def get_type(self) -> str:
        """Get the interface type.

        Possible values: "unknown", "loopback", "virtual", "ethernet", "wifi",
                         "can_bus", "fddi", "ppp", "slip", "phonet"
                         "ieee802154", "sixlowpan", "ieee80216", "ieee1394"

        Returns:
            interface type
        """
        return INTERFACE_TYPES.inv[self.type()]

    def get_address_entries(self) -> List[network.NetworkAddressEntry]:
        return [network.NetworkAddressEntry(i) for i in self.addressEntries()]

    @staticmethod
    def get_all_addresses() -> List[network.HostAddress]:
        return [network.HostAddress(i) for i in NetworkInterface.allAddresses()]

    @staticmethod
    def get_all_interfaces() -> List[network.HostAddress]:
        return [network.NetworkInterface(i) for i in NetworkInterface.allInterfaces()]

    @staticmethod
    def get_interface_from_name(name: str) -> NetworkInterface:
        interface = NetworkInterface.interfaceFromName(name)
        # if not interface:
        #     return None
        return NetworkInterface(interface)


if __name__ == "__main__":
    interface = NetworkInterface()
    print(bool(interface))



# -*- coding: utf-8 -*-

"""Charts module."""

from .barset import BarSet
from .boxset import BoxSet
from .candlestickset import CandlestickSet
from .pieslice import PieSlice
from .abstractaxis import AbstractAxis
from .valueaxis import ValueAxis
from .datetimeaxis import DateTimeAxis
from .categoryaxis import CategoryAxis
from .logvalueaxis import LogValueAxis
from .barcategoryaxis import BarCategoryAxis
from .abstractseries import AbstractSeries
from .abstractbarseries import AbstractBarSeries
from .barseries import BarSeries
from .percentbarseries import PercentBarSeries
from .horizontalbarseries import HorizontalBarSeries
from .horizontalpercentbarseries import HorizontalPercentBarSeries
from .stackedbarseries import StackedBarSeries
from .horizontalstackedbarseries import HorizontalStackedBarSeries
from .xyseries import XYSeries
from .scatterseries import ScatterSeries
from .lineseries import LineSeries
from .chart import Chart
from .polarchart import PolarChart
from .chartview import ChartView
from .legend import Legend

__all__ = [
    "BarSet",
    "BoxSet",
    "CandlestickSet",
    "PieSlice",
    "Legend",
    "AbstractSeries",
    "AbstractBarSeries",
    "BarSeries",
    "PercentBarSeries",
    "HorizontalBarSeries",
    "HorizontalPercentBarSeries",
    "StackedBarSeries",
    "HorizontalStackedBarSeries",
    "XYSeries",
    "AbstractAxis",
    "ValueAxis",
    "DateTimeAxis",
    "CategoryAxis",
    "LogValueAxis",
    "BarCategoryAxis",
    "ChartView",
    "Chart",
    "PolarChart",
    "LineSeries",
    "ScatterSeries",
]



# -*- coding: utf-8 -*-

from __future__ import annotations

from qtpy import QtWidgets


class TableWidgetSelectionRange(QtWidgets.QTableWidgetSelectionRange):
    def __repr__(self):
        return (
            f"{type(self).__name__}({self.topRow()}, {self.leftColumn()}, "
            f"{self.bottomRow()}, {self.rightColumn()})"
        )

    def __eq__(self, other: QtWidgets.QTableWidgetSelectionRange):
        return (
            self.topRow() == other.topRow()
            and self.bottomRow() == other.bottomRow()
            and self.leftColumn() == other.leftColumn()
            and self.rightColumn() == other.rightColumn()
        )

    def __or__(
        self, other: QtWidgets.QTableWidgetSelectionRange
    ) -> TableWidgetSelectionRange:
        result = TableWidgetSelectionRange(
            min(self.topRow(), other.topRow()),
            min(self.leftColumn(), other.leftColumn()),
            max(self.bottomRow(), other.bottomRow()),
            max(self.rightColumn(), other.rightColumn()),
        )
        return result

    def __and__(
        self, other: QtWidgets.QTableWidgetSelectionRange
    ) -> TableWidgetSelectionRange:
        if not (
            other.topRow() <= self.bottomRow()
            or other.bottomRow() >= self.topRow()
            or other.leftColumn() <= self.rightColumn()
            or other.rightColumn() >= self.leftColumn()
        ):
            return TableWidgetSelectionRange()
        result = TableWidgetSelectionRange(
            max(self.topRow(), other.topRow()),
            max(self.leftColumn(), other.leftColumn()),
            min(self.bottomRow(), other.bottomRow()),
            min(self.rightColumn(), other.rightColumn()),
        )
        return result


if __name__ == "__main__":
    range_1 = TableWidgetSelectionRange(0, 0, 10, 10)
    range_2 = TableWidgetSelectionRange(5, 5, 15, 15)
    result = range_1 == range_2
    print(result)
